import json

import Apifon.sdk.Mookee as mookee
from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Resource.Helpers.BounceReason import BounceReason
from Apifon.sdk.Resource.Helpers.BounceType import BounceType
from Apifon.sdk.Resource.Helpers.SortDirection import SortDirection
from Apifon.sdk.Model.Request.BounceQueryBuilder import BounceQueryBuilder
from Apifon.sdk.Model.Request.BounceRequestBuilder import BounceRequestBuilder
from Apifon.sdk.Model.Request.BounceSingleQuery import BounceSingleQuery
from Apifon.sdk.Resource.BounceListResource import BounceListResource


def remove_nones(var):
    for i in list(var):
        if var[i] is None or ((isinstance(var[i], dict) or isinstance(var[i], list)) and not bool(var[i])):
            var.pop(i)
        elif type(var[i]) is dict:
            remove_nones(var[i])
        elif type(var[i]) is list:
            var[i] = [x for x in var[i] if x != 20]
            for j in var[i]:
                if type(j) is dict:
                    remove_nones(j)


def print_json(output):
    json_string = json.dumps(output, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    json_dict = json.loads(json_string)
    remove_nones(json_dict)
    json_string = json.dumps(json_dict, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    print(json_string)


# This is a complete example on how to use the APIFON Subscriber SDK for Python.


Mookee = mookee.Mookee.get_instance()

'''
First things first, we need to set our credentials that we got from
the Mookee platform.
'''
Mookee.add_credentials("auth", "your_client_token", "your_secret_key")
Mookee.set_active_credential("auth")

'''
These are our resources
'''
bounceResource = BounceListResource()

choice = "1"

while choice != 0:
    print("------ Menu ------")
    print("0.  Exit")
    print("1.  Add destination to Block List")
    print("2.  Get Blocked Subscribers")
    print("3.  Get Blocked Subscribers with parameters")
    print("4.  Get Blocked Subscriber by destination")
    print("5.  Remove Blocked Subscriber")

    choice = input("Choice: ")
    try:
        choice = int(choice)
    except ValueError:
        print("Please enter an int.")

    '''
    Every request has the same structure: 
     *First we set the Credential we want to use. 
     *Then we build the components of the Request. 
     *Finally we build the Request and send it with the appropriate Resource. 
     *We print the response we get in console if applicable.
    '''

    # Exit Loop
    if choice == 0:
        print("Exiting...")

    # this request adds a destination to the block list
    elif choice == 1:
        '''
        You can add a subscribers phone number, email or both to the bounce list. Building the request requires
        the number or email, and the reason for blocking. Optionally you can add a description
        '''

        destination = input("Input a phone number or email to be added to the block list\n")

        bounceRequestBuilder = BounceRequestBuilder.get_bounce_request_builder(destination,
                                                                               BounceReason.sub_requested.value)
        bounceRequestBuilder.with_description("description as to why the subscriber was blocked")

        bounceRequest = bounceRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/bounce/subscriber/block")
        try:
            print("----- Bounce Request -----")
            print_json(bounceRequest)
            print("Sending request...")
            url = bounceResource.add_bounce_subscriber(bounceRequest)
            print("Subscriber added to bounce list. Fetching blocked subscriber...")
            bouncedSub = bounceResource.get_by_resource_uri(url)
            print("----- Bounced Subscriber -----")
            print_json(bouncedSub)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add Subscriber to Bounce list:\n" + str(x))

    # this request gets the bounce list
    elif choice == 2:

        '''
        Fetching bounce subscribers is always done in batches. The default batch number is 20.
        Sorting is also applied based on any of the fields. The default field is the bounced subscribers created date 
        and the default order is ascending.
        '''

        bounceQueryBuilder = BounceQueryBuilder.get_builder()
        bounceQuery = bounceQueryBuilder.build()

        print("Sending Request to Endpoint: /services/api/bounce/subscriber")
        try:
            print("Sending request...")
            bouncedSubs = bounceResource.get_bounce_subscribers(bounceQuery)
            print("----- Bounced Subscribers -----")
            print_json(bouncedSubs)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Bounced Subscribers:\n" + str(x))
        bounceResource.get_bounce_subscribers(bounceQuery)

    # this request gets the bounce list with parameters
    elif choice == 3:

        '''
        This request only gets the first three bounced subscribers with their destination being of type MOBILE
        and with the reason of 'Subscriber Requested'.They are sorted by their mobile in descending order. 
        The (partial) response will only fetch the destination, date created and type.
        '''
        fields = {"date_created", "destination", "type", }
        bounceQueryBuilder = BounceQueryBuilder.get_builder().with_type(BounceType.mobile.value) \
            .with_reason(BounceReason.sub_requested.value).add_visible_fields(fields).set_page(1) \
            .set_sort("destination", SortDirection.desc.value).set_size(3)

        bounceQuery = bounceQueryBuilder.build()

        print("Sending Request to Endpoint: /services/api/bounce/subscriber")
        try:
            print("Sending request...")
            bouncedSubs = bounceResource.get_bounce_subscribers(bounceQuery)
            print("----- Bounced Subscribers -----")
            print_json(bouncedSubs)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Bounced Subscribers:\n" + str(x))
        bounceResource.get_bounce_subscribers(bounceQuery)

    # This request gets a bounced subscriber by his destination
    elif choice == 4:

        destination = input("Input a phone number or email to be fetched from the block list\n")

        bounceSingleQuery = BounceSingleQuery.get_bounce_lookup(destination)

        print("Sending Request to Endpoint: /services/api/bounce/subscriber/{destination}")
        try:
            print("Fetching Bounced Subscriber...")
            theSub = bounceResource.get_bounce_subscriber(bounceSingleQuery)
            print("------ Subscriber ------")
            print_json(theSub)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Bounce Subscriber:\n" + str(x))

    # This request removes a subscriber from the bounced list
    elif choice == 5:

        destination = input("Input a phone number or email to be removed from the block list\n")

        print("Sending Request to Endpoint: /services/api/bounce/subscriber/{destination}")
        try:
            print("Removing Bounced Subscriber...")
            bounceResource.remove_bounce_subscriber(destination)
            print("Subscriber removed from bounce list")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to remove the Subscriber from the bounce list:\n" + str(x))
