import json
import Apifon.sdk.Mookee as mookee
from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Model.Request.GroupRequestBuilder import GroupRequestBuilder
from Apifon.sdk.Model.Request.GroupSingleQuery import GroupSingleQuery
from Apifon.sdk.Model.Request.SubgroupRequestBuilder import SubgroupRequestBuilder
from Apifon.sdk.Model.Request.SubgroupSingleQuery import SubgroupSingleQuery
from Apifon.sdk.Resource.GroupResource import GroupResource
from Apifon.sdk.Resource.Helpers.GroupOptionsOrientation import GroupOptionsOrientation
from Apifon.sdk.Resource.Helpers.GroupType import GroupType
from Apifon.sdk.Resource.Helpers.ListType import ListType
from Apifon.sdk.Resource.Helpers.SelectionType import SelectionType
from Apifon.sdk.Resource.Helpers.SortDirection import SortDirection
from Apifon.sdk.Resource.Helpers.SubscriberFieldType import SubscriberFieldType
from Apifon.sdk.Model.Request.CampaignDefaultsBuilder import CampaignDefaultsBuilder
from Apifon.sdk.Model.Request.GdprFieldRequestBuilder import GdprFieldRequestBuilder
from Apifon.sdk.Model.Request.GdprPropertiesBuilder import GdprPropertiesBuilder
from Apifon.sdk.Model.Request.ListQueryBuilder import ListQueryBuilder
from Apifon.sdk.Model.Request.ListRequestBuilder import ListRequestBuilder
from Apifon.sdk.Model.Request.ListSingleQuery import ListSingleQuery
from Apifon.sdk.Model.Request.SubscriberFieldRequestBuilder import SubscriberFieldRequestBuilder
from Apifon.sdk.Resource.GdprFieldResource import GdprFieldResource
from Apifon.sdk.Resource.ListResource import ListResource
from Apifon.sdk.Resource.SubgroupResource import SubgroupResource
from Apifon.sdk.Resource.SubscriberFieldResource import SubscriberFieldResource


def remove_nones(var):
    for i in list(var):
        if var[i] is None or ((isinstance(var[i], dict) or isinstance(var[i], list)) and not bool(var[i])):
            var.pop(i)
        elif type(var[i]) is dict:
            remove_nones(var[i])
        elif type(var[i]) is list:
            var[i] = [x for x in var[i] if x != 20]
            for j in var[i]:
                if type(j) is dict:
                    remove_nones(j)


def print_json(output):
    json_string = json.dumps(output, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    json_dict = json.loads(json_string)
    remove_nones(json_dict)
    json_string = json.dumps(json_dict, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    print(json_string)


# This is a complete example on how to use the APIFON List SDK for Python.


Mookee = mookee.Mookee.get_instance()

'''
First things first, we need to set our credentials that we got from
the Mookee platform.
'''
Mookee.add_credentials("auth", "your_client_token", "your_secret_key")
Mookee.set_active_credential("auth")

'''
These are our resources
'''
listResource = ListResource()
subscriberFieldResource = SubscriberFieldResource()
gdprFieldResource = GdprFieldResource()
groupResource = GroupResource()
subgroupResource = SubgroupResource()

defaultListId = "your_default_list"
defaultSubscriberFieldField = "your_default_subscriber_field"
defaultGdprFieldField = "your_default_gdpr_field"
defaultGroupField = "your_default_group_field"
defaultOptionsField = "your_default_subgroup_field"

choice = "1"

while choice != 0:
    print("------ Menu ------")
    print("0.  Exit")
    print("1.  Create Simple List")
    print("2.  Create Complete List")
    print("3.  Get Lists")
    print("4.  Get Lists with parameters")
    print("5.  Get Single List")
    print("6.  Update List")
    print("7.  Delete List")
    print("8:  Add Subscriber Field")
    print("9.  Update Subscriber Field")
    print("10. Remove Subscriber Field")
    print("11. Add GDPR Field")
    print("12: Update GDPR Field")
    print("13: Add Simple Group")
    print("14: Add Complete Group")
    print("15: Get Group")
    print("16: Update Group")
    print("17: Remove Group")
    print("18: Add Subgroup to Group")
    print("19: Add Subgroups to Group")
    print("20: Get Subgroups from Group")
    print("21: Update Subgroup")
    print("22: Remove Subgroup")
    choice = input("Choice: ")
    try:
        choice = int(choice)
    except ValueError:
        print("Please enter an int.")

    '''
    Every request has the same structure: 
     *First we set the Credential we want to use. 
     *Then we build the components of the Request. 
     *Finally we build the Request and send it with the appropriate Resource. 
     *We print the response we get in console if applicable.
    '''
    # Exit Loop
    if choice == 0:
        print("Exiting...")

    # This is a simple List creation.
    elif choice == 1:

        '''
        A simple List refers to the creation of a list with the least possible mandatory fields.
        These values are list type, title and prefix.
        '''

        listRequestBuilder = ListRequestBuilder.get_add_request_builder(ListType.advanced.value, "My Simple List", '30')
        listRequest = listRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list")
        print("------ List Request Body ------")
        print_json(listRequest)
        try:
            listUrl = listResource.create_list(listRequest)
            print("Fetching New List...")
            newList = listResource.get_by_resource_uri(listUrl)
            print("------ New List ------")
            print_json(newList)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to create Simple List:\n" + str(x))

    # This is a complete List creation.
    elif choice == 2:

        '''
        A complete List contains all possible fields.These are title, list type, prefix,description,
         campaign defaults and gdpr properties
        '''

        listRequestBuilder = ListRequestBuilder.get_add_request_builder(ListType.advanced.value, "My Complete List",
                                                                        '30')
        listRequestBuilder.with_description("Custom description of List")

        campaignDefaults = CampaignDefaultsBuilder()
        campaignDefaults.sender_name = "Email name"
        campaignDefaults.from_ = "your_email@email.com"
        campaignDefaults.reply_to = "someones_email@email.com"
        campaignDefaults.sender_id = "SMS Name"
        campaignDefaults.viber_sender_id = "Apifon"
        listRequestBuilder.with_campaign_defaults(campaignDefaults)

        gdprProperties = GdprPropertiesBuilder()
        gdprProperties.gdpr_compliant = True
        gdprProperties.title = "my gdpr properties title"
        gdprProperties.description = "short description"
        gdprProperties.legal_text = "legal text"
        listRequestBuilder.with_gdpr_properties(gdprProperties)

        listRequest = listRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list")
        print("------ List Request Body ------")
        print_json(listRequest)
        try:
            listUrl = listResource.create_list(listRequest)
            print("Fetching New List...")
            newList = listResource.get_by_resource_uri(listUrl)
            print("------ New List ------")
            print_json(newList)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to create Complete List:\n" + str(x))

    # This will fetch lists
    elif choice == 3:

        '''
        Fetching Lists is always done in batches. The default batch number is 20.
        Sorting is also applied based on any of the fields. The default field is the Lists's id 
        and the default order is ascending.
        '''

        listQueryBuilder = ListQueryBuilder().get_builder()
        listQuery = listQueryBuilder.build()

        print("Sending Request to Endpoint: /services/api/list")
        try:
            allLists = listResource.get_lists(listQuery)
            print("Fetching Lists...")
            print_json(allLists)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Lists:\n" + str(x))


    # This will fetch lists with parameters
    elif choice == 4:

        '''
        This request only gets the first five lists with type 'advanced', sorted by their title in ascending order. 
        The (partial) response will only bring the titles, subscriber fields, total number of subscribers
        and the dates the lists were created  
        '''
        listQueryBuilder = ListQueryBuilder().get_builder()
        listQueryBuilder.with_type(ListType.advanced.value).set_page(1).set_size(5).set_sort("title",
                                                                                             SortDirection.asc.value)

        fields = {"title", "subscriber_fields", "total_subscribers", "date_created"}
        listQueryBuilder.add_visible_fields(fields)

        listQuery = listQueryBuilder.build()

        print("Sending Request to Endpoint: /services/api/list")
        try:
            allLists = listResource.get_lists(listQuery)
            print("Fetching Lists...")
            print_json(allLists)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Lists:\n" + str(x))

    # this will fetch one list
    elif choice == 5:
        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        listSinglQuery = ListSingleQuery.get_list_request(listId)

        print("Sending Request to Endpoint: /services/api/list/{list_id}")
        try:
            theList = listResource.get_list(listSinglQuery)
            print("Fetching List...")
            print_json(theList)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the List:\n" + str(x))

    # This updates a list
    elif choice == 6:

        '''
        Updating a List works similar to adding one. You have to build a List request acting as the update
        bundle. Any fields omitted or set to 'None' are marked for deletion. Which is why the most convenient way of 
        updating is first fetching the List you want updated and passing it to the builder.
        After which you may make any necessary changes. This is demonstrated bellow.

        In this example we change the title, description and the gdpr properties title and legal text
        as well as the 'from' field of the campaign defaults
        '''

        theList = None

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        # we first fetch the list we want to update
        listSinglQuery = ListSingleQuery.get_list_request(listId)
        print("Sending Request to Endpoint: /services/api/list/{list_id}")

        try:
            theList = listResource.get_list(listSinglQuery)
            print("Fetching List...")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the List:\n" + str(x))

        if theList is not None:

            updateListRequestBuilder = ListRequestBuilder.get_update_list_request_builder(theList)
            updateListRequestBuilder.with_title("My updated List").with_description("This list has been updated")

            gdprProperties = updateListRequestBuilder.gdpr_properties
            gdprProperties.title = "My updated gdpr properties title"
            gdprProperties.legal_text = "My updated legal text"
            updateListRequestBuilder.with_gdpr_properties(gdprProperties)

            updateListRequestBuilder.campaign_defaults.from_ = "updated_email@email.com"

            updateListRequest = updateListRequestBuilder.build()

            print("Sending Request to Endpoint: /services/api/list/{list_id}")
            print("------Updated List Request Body ------")
            print_json(updateListRequest)
            try:
                listUrl = listResource.update_list(updateListRequest)
                print("Fetching Updated List...")
                updatedList = listResource.get_by_resource_uri(listUrl)
                print("------ Updated List ------")
                print_json(updatedList)
            except ApifonRestError as x:
                print("Apifon Rest Error while trying to updated the List:\n" + str(x))

    # this deletes a list
    elif choice == 7:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        print("Sending Request to Endpoint: /services/api/list/{list_id}")
        try:
            listResource.remove_list(listId)
            print("List deleted successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to delete the List:\n" + str(x))

    # this adds a subscriber field to a list
    elif choice == 8:
        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        subscriberFieldRequestBuilder = SubscriberFieldRequestBuilder. \
            get_add_request_builder(listId, SubscriberFieldType.text.value, 20, "new subscriber field")

        subscriberFieldRequest = subscriberFieldRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/field")
        try:
            subscriberFieldResource.add_subscriber_field(subscriberFieldRequest)
            print("Field added successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add the subscriber field:\n" + str(x))

    # this updates a subscriber field
    elif choice == 9:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        subField = input("Press D to use the default subscriber-field field. Otherwise input a field\n")
        if subField == "D" or subField == "d":
            subField = defaultSubscriberFieldField

        subscriberFieldRequestBuilder = SubscriberFieldRequestBuilder.get_update_subscriber_field_request_builder(listId, subField, "updated name", 40)

        subscriberFieldRequest = subscriberFieldRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/field")
        try:
            subscriberFieldResource.update_subscriber_field(subscriberFieldRequest)
            print("Field updated successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to update the subscriber field:\n" + str(x))

    # this removes a subscriber field
    elif choice == 10:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        subField = input("Press D to use the default subscriber-field field. Otherwise input a field\n")
        if subField == "D" or subField == "d":
            subField = defaultSubscriberFieldField

        print("Sending Request to Endpoint: /services/api/list/{list_id}/field/{field}")
        try:
            subscriberFieldResource.remove_subscriber_field(listId, subField)
            print("Subscriber field removed successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to remove the subscriber field:\n" + str(x))

    # this adds a gdpr field
    elif choice == 11:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        gdprFieldRequestBuilder = GdprFieldRequestBuilder.get_add_gtpr_field_request_builder("new gdpr field", listId)

        gdprFieldRequest = gdprFieldRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/gdpr")
        try:
            gdprFieldResource.add_gdpr_field(gdprFieldRequest)
            print("Field added successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add gdpr field:\n" + str(x))


    # this updates a gdpr field
    elif choice == 12:
        '''
        This updates the title, and the description of the gdpr field
        '''

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        gdprField = input("Press D to use the default gdpr-field field. Otherwise input a field\n")
        if gdprField == "D" or gdprField == "d":
            gdprField = defaultGdprFieldField

        gdprFieldRequestBuilder = GdprFieldRequestBuilder.get_update_gdpr_field_request_builder("updated gdpr field", gdprField, listId)
        gdprFieldRequestBuilder.with_description("updated description")

        gdprFieldRequest = gdprFieldRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/gdpr/{field}")
        try:
            gdprFieldResource.update_gdpr_field(gdprFieldRequest)
            print("Field updated successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to update gdpr field:\n" + str(x))

    # this adds a simple group
    elif choice == 13:
        ''''
        A simple group refers to the creation of a group with the least mandatory fields. 
        These values are name and type
        '''

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupRequestBuilder = GroupRequestBuilder.get_add_request_builder("simple group", GroupType.group.value, listId)
        groupRequest = groupRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/group")
        print("------ Group Request Body ------")
        print_json(groupRequest)
        try:
            groupUrl = groupResource.add_group(groupRequest)
            print("Fetching new Group...")
            newGroup = groupResource.get_by_resource_uri(groupUrl)
            print("------ New Group ------")
            print_json(newGroup)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add group:\n" + str(x))

    # this adds a complete group
    elif choice == 14:

        '''' A complete group contains all possible fields.These are name, type, internal, selection type and options 
        orientation. 
        '''
        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupRequestBuilder = GroupRequestBuilder.get_add_request_builder("complete group", GroupType.group.value,
                                                                          listId)
        groupRequestBuilder.with_internal(False).with_selectionType(SelectionType.single.value) \
            .with_groupOptionsOrientation(GroupOptionsOrientation.horizontal.value)

        groupRequest = groupRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/group")
        print("------ Group Request Body ------")
        print_json(groupRequest)
        try:
            groupUrl = groupResource.add_group(groupRequest)
            print("Fetching new Group...")
            newGroup = groupResource.get_by_resource_uri(groupUrl)
            print("------ New Group ------")
            print_json(newGroup)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add group:\n" + str(x))

    # this will fetch one group
    elif choice == 15:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        groupSingleQuery = GroupSingleQuery.get_group_request(listId,groupField)

        print("Sending Request to Endpoint: /services/api/list/{list_id}/group/{group_field}")
        try:
            print("Fetching Group...")
            theGroup = groupResource.get_group(groupSingleQuery)
            print_json(theGroup)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to fetch the group:\n" + str(x))

    # This updates a group
    elif choice == 16:

        '''
        Updating a group works similar to adding one. You have to build a Group request acting as the update
        bundle.The most convenient way of updating is first fetching the Group you want updated and passing it to the
        builder. After which you may make any necessary changes. This is demonstrated bellow.

        In this example we change the name, group options orientation and selection_type
        '''

        theGroup = None

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        # we first fetch the group we want to update
        groupSingleQuery = GroupSingleQuery(listId, groupField)

        print("Sending Request to Endpoint: /services/api/list/{list_id}/group/{group_field}")
        try:
            print("Fetching Group...")
            theGroup = groupResource.get_group(groupSingleQuery)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to fetch the group:\n" + str(x))

        if theGroup is not None:

            updateGroupRequestBuilder = GroupRequestBuilder.get_update_request_builder_from_response(listId, theGroup)
            updateGroupRequestBuilder.with_name("updated group name 2") \
                .with_groupOptionsOrientation(GroupOptionsOrientation.horizontal.value) \
                .with_selectionType(SelectionType.multiple.value)

            updateVGroupRequest = updateGroupRequestBuilder.build()

            print("Sending Request to Endpoint: /services/api/list/{list_id}/group/{group_field}")
            print("------Updated Group Request Body ------")
            try:
                print_json(updateGroupRequestBuilder)
                listUrl = groupResource.update_group(updateVGroupRequest)
                print("Fetching Updated Group...")
                updatedGroup = groupResource.get_by_resource_uri(listUrl)
                print("------ Updated Group ------")
                print_json(updatedGroup)
            except ApifonRestError as x:
                print("Apifon Rest Error while trying to updated the Group:\n" + str(x))

    # this deletes a group
    elif choice == 17:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        print("Sending Request to Endpoint: /services/api/list/{list_id}/group/{group_field}")
        try:
            if groupResource.remove_group(listId, groupField):
                print("Group removed successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to delete the Group:\n" + str(x))

        # this adds a subgroup
    elif choice == 18:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        subgroupRequestBuilder = SubgroupRequestBuilder.get_add_request_builder("option1", listId, groupField)
        subgroupRequest = subgroupRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/{group_field}/option")
        try:
            subgroupUrl = subgroupResource.add_subgroup(subgroupRequest)
            print("Fetching new Subgroup...")
            newSubgroup = subgroupResource.get_by_resource_uri(subgroupUrl)
            print("------ New Subgroup ------")
            print_json(newSubgroup)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add subgroup:\n" + str(x))

    # this adds many subgroups
    elif choice == 19:
        '''
        You also have the option of adding multiple options in one request
        '''

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        myOptions = ['option11', 'option21']
        subgroupRequestBuilder = SubgroupRequestBuilder.get_add_multiple_request_builder(myOptions, listId, groupField)
        subgroupRequest = subgroupRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/{group_field}/options")
        try:
            subgroupUrl = subgroupResource.add_subgroups(subgroupRequest)
            print("Subgroups added successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add subgroups:\n" + str(x))

    # this gets a subgroup
    elif choice == 20:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        optionField = input("Press D to use the default option field. Otherwise input an option field\n")
        if optionField == "D" or optionField == "d":
            optionField = defaultOptionsField

        subgroupSingleQuery = SubgroupSingleQuery.get_subgroup_request(listId,groupField,optionField)

        print("Sending Request to Endpoint: /services/api/list/{list_id}/{group_field}/option/{option_field}")
        try:
            theSubgroup = subgroupResource.get_subgroup(subgroupSingleQuery)
            print("------ The Subgroup ------")
            print_json(theSubgroup)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the subgroup:\n" + str(x))

    # this updates a subgroup
    elif choice == 21:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        optionField = input("Press D to use the default subgroup field. Otherwise input a subgroup field\n")
        if optionField == "D" or optionField == "d":
            optionField = defaultOptionsField

        subgroupRequestBuilder = SubgroupRequestBuilder.get_update_request_builder("updated option", listId, groupField,
                                                                                   optionField)
        subgroupRequest = subgroupRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{list_id}/{group_field}/{option_field}")
        try:
            subgroupUrl = subgroupResource.update_subgroup(subgroupRequest)
            print("Subgroup updated successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to update subgroup:\n" + str(x))

    # this deletes a subgroup
    elif choice == 22:

        listId = input("Press D to use the default list id. Otherwise input a list id\n")
        if listId == "D" or listId == "d":
            listId = defaultListId

        groupField = input("Press D to use the default group field. Otherwise input a group field\n")
        if groupField == "D" or groupField == "d":
            groupField = defaultGroupField

        optionField = input("Press D to use the default option field. Otherwise input an option field\n")
        if optionField == "D" or optionField == "d":
            optionField = defaultOptionsField

        print("Sending Request to Endpoint: /services/api/list/{list_id}/{group_field}/{option_field}")
        try:
            print("Deleting Subgroup...")
            if subgroupResource.remove_subgroup(listId, groupField, optionField):
                print("Subgroup deleted successfully")

        except ApifonRestError as x:
            print("Apifon Rest Error while trying to delete the subgroup:\n" + str(x))
