import json
import Apifon.sdk.Mookee as mookee
from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Resource.Helpers.Condition import Condition
from Apifon.sdk.Resource.Helpers.ConditionOperator import ConditionOperator
from Apifon.sdk.Resource.Helpers.SortDirection import SortDirection
from Apifon.sdk.Resource.Helpers.SubscriberStatus import SubscriberStatus
from Apifon.sdk.Model.Request.SubscriberConditionQueryRequestBuilder import SubscriberConditionQueryRequestBuilder
from Apifon.sdk.Model.Request.SubscriberQueryBuilder import SubscriberQueryBuilder
from Apifon.sdk.Model.Request.SubscriberRequestBuilder import SubscriberRequestBuilder
from Apifon.sdk.Model.Request.SubscriberSingleQuery import SubscriberSingleQuery
from Apifon.sdk.Resource.SubscriberResource import SubscriberResource


def remove_nones(var):
    for i in list(var):
        if var[i] is None or ((isinstance(var[i], dict) or isinstance(var[i], list)) and not bool(var[i])):
            var.pop(i)
        elif type(var[i]) is dict:
            remove_nones(var[i])
        elif type(var[i]) is list:
            var[i] = [x for x in var[i] if x != 20]
            for j in var[i]:
                if type(j) is dict:
                    remove_nones(j)


def print_json(output):
    json_string = json.dumps(output, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    json_dict = json.loads(json_string)
    remove_nones(json_dict)
    json_string = json.dumps(json_dict, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    print(json_string)


# This is a complete example on how to use the APIFON Subscriber SDK for Python.


Mookee = mookee.Mookee.get_instance()

'''
First things first, we need to set our credentials that we got from
the Mookee platform.
'''
Mookee.add_credentials("auth", "your_client_token", "your_secret_key")
Mookee.set_active_credential("auth")

'''
These are our resources
'''

subscriberResource = SubscriberResource()

defaultListId = "your default list id"
defaultSubscriberLookupMD5 = "your default subscriber lookupMD5"

choice = "1"

while choice != 0:
    print("------ Menu ------")
    print("0.  Exit")
    print("1.  Add Simple Subscriber")
    print("2.  Add Complete Subscriber")
    print("3.  Get Subscribers")
    print("4.  Get Subscribers with parameters")
    print("5.  Query Subscribers")
    print("6.  Get Single Subscriber")
    print("7.  Update Subscriber")
    print("8:  Remove Subscriber")
    print("9.  Subscribe Subscriber")
    print("10. Unsubscribe Subscriber")

    choice = input("Choice: ")
    try:
        choice = int(choice)
    except ValueError:
        print("Please enter an int.")

    '''
    Every request has the same structure: 
     *First we set the Credential we want to use. 
     *Then we build the components of the Request. 
     *Finally we build the Request and send it with the appropriate Resource. 
     *We print the response we get in console if applicable.
    '''

    '''
    The examples use the default list id declared above.
    '''

    # Exit Loop
    if choice == 0:
        print("Exiting...")

    # this adds a simple subscriber
    elif choice == 1:
        '''
        A simple subscriber is a subscriber addition with the least possible mandatory fields.
        The subscriber needs either a mobile phone number or an email. This example uses a phone number
        '''

        subscriberRequestBuilder = SubscriberRequestBuilder.get_add_subscriber_request_builder(defaultListId)
        subscriberRequestBuilder.with_destination("699999999999", SubscriberStatus.subscribed.value)

        subscriberRequest = subscriberRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber")
        print("------ Subscriber Request Body ------")
        print_json(subscriberRequest)
        try:
            subUrl = subscriberResource.add_subscriber(subscriberRequest)
            print("Fetching New Subscriber...")
            newSub = subscriberResource.get_by_resource_uri(subUrl)
            print("------ New Subscriber ------")
            print_json(newSub)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to add a Simple Subscriber:\n" + str(x))

    # this adds a complete subscriber
    elif choice == 2:
        '''A complete subscriber contains all possible fields.These are destination,email, subscriber fields, 
        gdpr fields and group options. Note: the subscriber, gdpr fields and group options need to exist as part of 
        the list in order for the addition to work '''

        subscriberRequestBuilder = SubscriberRequestBuilder.get_add_subscriber_request_builder(defaultListId)
        subscriberRequestBuilder.with_destination("699999999999", SubscriberStatus.subscribed.value) \
            .with_email("subscriber@email.com", SubscriberStatus.subscribed.value) \
            .with_subscriber_field("FIRST_NAME", "subscriber name") \
            .with_subscriber_field("LAST_NAME", "subscriber surname") \
            .with_gdpr_field("GDPR_FIELD_1", True) \
            .with_mobile_gdpr_field("GDPR_FIELD_2", True)

        myOptions = ["OPTION_THREE_b5499fe9-fae8-433e-a07a-2555c80db493"]
        subscriberRequestBuilder.with_groupOptions(myOptions)

        subscriberRequest = subscriberRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber")
        print("------ Subscriber Request Body ------")
        print_json(subscriberRequest)
        try:
            subUrl = subscriberResource.add_subscriber(subscriberRequest)
            print("Fetching New Subscriber...")
            newSub = subscriberResource.get_by_resource_uri(subUrl)
            print("------ New Subscriber ------")
            print_json(newSub)
        except ApifonRestError as x:
                print("Apifon Rest Error while trying to add a Complete Subscriber:\n" + str(x))

    # this request gets subscribers
    elif choice == 3:
        '''
        Fetching subscribers is always done in batches. The default batch number is 20.
        Sorting is also applied based on any of the fields. The default field is the subscriber's id 
        and the default order is ascending.
        '''

        subscriberQueryBuilder = SubscriberQueryBuilder.get_query_builder(defaultListId)

        subscriberQuery = subscriberQueryBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber")
        try:
            print("Fetching Subscribers...")
            theSubs = subscriberResource.querry_subscribers(subscriberQuery)
            print("------ Subscribers ------")
            print_json(theSubs)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Subscribers:\n" + str(x))

    # this request gets subscribers while having set all possible parameters
    elif choice == 4:
        '''
        This request only gets the first two subscribers with a mobile status of SUBSCRIBED and an email status of UNSUBSCRIBED
        sorted by their first name (this subscriber field must exist in order for the request to work) in descending 
        order. The (partial) response will only fetch the subscriber fields, destination (phone number), email and id.
        '''
        fields = {"subscriber_fields", "destination", "email", "id"}

        subscriberQueryBuilder = SubscriberQueryBuilder.get_query_builder(defaultListId)
        subscriberQueryBuilder.with_status(SubscriberStatus.subscribed.value) \
            .with_email_status(SubscriberStatus.unsubscribed.value).set_size(2) \
            .set_sort("subscriber_fields.FIRST_NAME", SortDirection.desc.value).set_page(1) \
            .add_visible_fields(fields)

        subscriberQuery = subscriberQueryBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber")
        try:
            print("Fetching Subscribers...")
            theSubs = subscriberResource.querry_subscribers(subscriberQuery)
            print("------ Subscribers ------")
            print_json(theSubs)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Subscribers:\n" + str(x))

    # this request applies conditions for fetching precise subscriber bundles
    elif choice == 5:
        '''
        The API provides a way of fetching precise subscriber bundles by setting some Conditions.
        These conditions take as parameters the field you want a condition applied to, the condition operator
        applied to the field, and the value that the field will be compared to according to the operator.
        
        In this example we set two conditions. The first checks if the field 'destination' exists, and the second 
        checks if the subscriber field 'FIRST_NAME' begins with the letters 'su'. You may build a Condition(s) and 
        then add it/them to the request or you may use the request's 'with_condition_parameters' function to add the 
        3 parameters. Both ways are demonstrated bellow. 
        
        '''

        condition1 = Condition("destination", ConditionOperator.exists.value, "")

        subscriberConditionRequestBuilder = SubscriberConditionQueryRequestBuilder.get_subscriber_conditions_builder(
            defaultListId)
        subscriberConditionRequestBuilder.with_condition(condition1) \
            .with_condition_parameters("subscriber_fields.FIRST_NAME", ConditionOperator.beginsWith.value, "Su")

        subscriberQuery = subscriberConditionRequestBuilder.build()

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber")
        try:
            print("Fetching Subscribers...")
            theSubs = subscriberResource.querry_subscribers_with_conditions(subscriberQuery)
            print("------ Subscribers ------")
            print_json(theSubs)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Subscribers:\n" + str(x))

    # this request gets a single subscriber
    elif choice == 6:

        '''
        You can directly fetch one subscriber if you know his destination or email id
        '''

        subLookupMD5 = input(
            "Press D to use the default destination id. Otherwise input a destination id or email id\n")
        if subLookupMD5 == "D" or subLookupMD5 == "d":
            subLookupMD5 = defaultSubscriberLookupMD5

        subscriberSingleQuery = SubscriberSingleQuery.get_subscriber_lookup(subLookupMD5, defaultListId)

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber/{lookupMD5}")
        try:
            print("Fetching Subscriber...")
            theSub = subscriberResource.get_subscriber(subscriberSingleQuery)
            print("------ Subscriber ------")
            print_json(theSub)
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Subscriber:\n" + str(x))

    # this request updates a subscriber
    elif choice == 7:

        '''
        Updating a subscriber works similar to adding one. You have to build a subscriber request acting as the update
        bundle. Any fields omitted or set to 'None' are marked for deletion. Which is why the most convenient way of 
        updating is first fetching the subscriber you want updated and passing him to the builder.
        After which you may make any necessary changes. This is demonstrated bellow.
        
        You can change every field besides the destination and email, unless they are empty. In this example we change
        the status, the email_status, the available subscriber fields and gdpr field subscriptions.
            '''
        subToUpdate = None

        subLookupMD5 = input(
            "Press D to use the default destination id. Otherwise input a destination id or email id\n")
        if subLookupMD5 == "D" or subLookupMD5 == "d":
            subLookupMD5 = defaultSubscriberLookupMD5

        subscriberSingleQuery = SubscriberSingleQuery.get_subscriber_lookup(subLookupMD5, defaultListId)

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber/{lookupMD5}")
        try:
            print("Fetching Subscriber...")
            subToUpdate = subscriberResource.get_subscriber(subscriberSingleQuery)
            print("------ Subscriber ------")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to get the Subscriber:\n" + str(x))

        if subToUpdate is not None:

            subscriberRequestBuilder = SubscriberRequestBuilder.get_builder_from_subscriber(subToUpdate)
            subscriberRequestBuilder.with_email_status(SubscriberStatus.subscribed.value) \
                .with_status(SubscriberStatus.unsubscribed.value) \
                .with_gdpr_field("GDPR_FIELD_1", True) \
                .with_subscriber_field("FIRST_NAME", "updated name") \
                .with_subscriber_field("LAST_NAME", "updated surname")

            subscriberUpdateRequest = subscriberRequestBuilder.build()

            print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber/{lookupMD5}")
            try:
                print("------ Subscriber Update Request ------")
                print_json(subscriberUpdateRequest)
                print("Updating Subscriber...")
                url = subscriberResource.update_subscriber(subscriberUpdateRequest)
                print("Subscriber updated. Fetching subscriber...")
                updatedSub = subscriberResource.get_by_resource_uri(url)
                print("------ Updated Subscriber ------")
                print_json(updatedSub)
            except ApifonRestError as x:
                print("Apifon Rest Error while trying to update the Subscriber:\n" + str(x))

    # this request removes a subscriber
    elif choice == 8:
        subLookupMD5 = input(
            "Press D to use the default destination id. Otherwise input a destination id or email id\n")
        if subLookupMD5 == "D" or subLookupMD5 == "d":
            subLookupMD5 = defaultSubscriberLookupMD5

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber/{lookupMD5}")
        try:
            print("Removing Subscriber...")
            subscriberResource.remove_subscriber(defaultListId, subLookupMD5)
            print("Subscriber removed successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to remove the Subscriber:\n" + str(x))

    # this request subscribes a subscriber's destination
    elif choice == 9:
        subLookupMD5 = input(
            "Press D to use the default destination id. Otherwise input a destination id or email id\n")
        if subLookupMD5 == "D" or subLookupMD5 == "d":
            subLookupMD5 = defaultSubscriberLookupMD5

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber/{lookupMD5}")
        try:
            print("Subscribing Subscriber...")
            subscriberResource.subscribe_subscriber(defaultListId, subLookupMD5)
            print("Subscriber subscribed successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to subscribe the Subscriber:\n" + str(x))

    # this request un-subscribes a subscriber's destination
    elif choice == 10:
        subLookupMD5 = input(
            "Press D to use the default destination id. Otherwise input a destination id or email id\n")
        if subLookupMD5 == "D" or subLookupMD5 == "d":
            subLookupMD5 = defaultSubscriberLookupMD5

        print("Sending Request to Endpoint: /services/api/list/{listId}/subscriber/{lookupMD5}")
        try:
            print("Unsubscribing Subscriber...")
            subscriberResource.unsubscribe_subscriber(defaultListId, subLookupMD5)
            print("Subscriber unsubscribed successfully")
        except ApifonRestError as x:
            print("Apifon Rest Error while trying to unsubscribe the Subscriber:\n" + str(x))
