from urllib.parse import urlencode

from Apifon.sdk.Resource.Helpers.AbstractFieldSerializable import AbstractFieldSerializable
from Apifon.sdk.Resource.Helpers.BounceReason import BounceReason
from Apifon.sdk.Resource.Helpers.BounceType import BounceType
from Apifon.sdk.Resource.Helpers.PaginatedRequest import PaginatedRequest
from Apifon.sdk.Resource.Helpers.PaginatedRequestBuilder import PaginatedRequestBuilder


class BounceQueryBuilder(PaginatedRequestBuilder, AbstractFieldSerializable):

    def __init__(self):
        super().__init__()
        self.type = None
        self.reason = None
        self.fields = set()

    @staticmethod
    def get_builder():
        return BounceQueryBuilder()

    # type is BounceType.value
    def with_type(self, type_):
        if type_ is None or not BounceType.has_value(type_):
            raise TypeError("Invalid type. Should be of type BounceType.value")
        self.type = type_
        return self

    # reason is BounceReason.value
    def with_reason(self, reason):
        if reason is None or not BounceReason.has_value(reason):
            raise TypeError("Invalid reason. Should be of type BounceReason.value")
        self.reason = reason
        return self

    def add_visible_fields(self, fields):
        if fields is None or not isinstance(fields, set):
            raise TypeError("Invalid fields. Should be of type Set ")
        self.fields.update(fields)
        return self

    def add_visible_field(self, field):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        self.fields.add(field)
        return self

    def get_fields(self):
        return self.fields

    def get_path_id(self):
        raise NotImplementedError("Not supported yet.")

    def to_query_param_request(self):
        self.build().to_query_param_request()

    def build(self):
        return self.QueryBounceSubscribersRequest(self.type, self.reason, self.fields, self.size,
                                                  self.page, self.sort, self.sort_dir)

    class QueryBounceSubscribersRequest(PaginatedRequest):
        def __init__(self, type_, reason, fields, size, page, sort_field, sort_dir):
            super().__init__(page, size, sort_field, sort_dir)
            self.type = type_
            self.reason = reason
            self.fields = fields

        def to_query_param_request(self):
            queryParams = {}
            self.add_query_params(queryParams)
            if self.type is not None and BounceType.has_value(self.type):
                queryParams["type"] = self.type
            if self.reason is not None and BounceReason.has_value(self.reason):
                queryParams["reason"] = self.reason
            if self.fields is not None and bool(self.fields):
                queryParams['fields'] = ','.join(self.fields)
            return '?' + urlencode(queryParams)
