import json

from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable
from Apifon.sdk.Resource.Helpers.BounceReason import BounceReason


class BounceRequestBuilder:

    def __init__(self, destination, reason):
        self.destination = destination
        self.reason = reason
        self.description = None

    # reason is BounceReason.value
    @staticmethod
    def get_bounce_request_builder(destination, reason):
        if destination is None or not isinstance(destination, str):
            raise TypeError("Invalid destination. Should be of type str")
        if reason is None or not BounceReason.has_value(reason):
            raise TypeError("Invalid reason. Should be of type BounceReason.value")
        return BounceRequestBuilder(destination, reason)

    def with_description(self, description):
        if description is None or not isinstance(description, str):
            raise TypeError("Invalid description. Should be of type str")
        self.description = description
        return self

    def build(self):
        return BounceRequestBuilder.BounceRequest(self.destination, self.reason, self.description)

    class BounceRequest(AbstractJsonSerializable):
        def __init__(self, destination, reason, description):
            self.destination = destination
            self.reason = reason
            self.description = description

        # override
        def get_body(self):

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)
            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
