import json

from Apifon.sdk.Resource.Helpers.AbstractHttpEtag import AbstractHttpEtag
from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable


class GdprFieldRequestBuilder:

    def __init__(self, list_id, name, field=None):
        self.list_id = list_id
        self.field = field
        self.name = name
        self.description = None
        self.etag = None

    @staticmethod
    def get_update_gdpr_field_request_builder(name, field, list_id):
        if field is None or not isinstance(field, str):
            raise TypeError("Incorrect field. Should be of type str")
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Incorrect list id. Should be of type str")

        gdpr_field_builder = GdprFieldRequestBuilder(list_id, name, field)
        return gdpr_field_builder

    @staticmethod
    def get_add_gtpr_field_request_builder(name, list_id):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Incorrect list id. Should be of type str")
        if name is None or not isinstance(name, str):
            raise TypeError("Incorrect name. Should be of type str")

        gdpr_field_builder = GdprFieldRequestBuilder(list_id, name)
        return gdpr_field_builder

    def with_description(self, description):
        if description is None or not isinstance(description, str):
            raise TypeError("Incorrect description. Should be of type str")

        self.description = description
        return self

    def with_etag(self, etag):
        if etag is None or not isinstance(etag, str):
            raise TypeError("Incorrect etag. Should be of type str")
        self.etag = etag
        return self

    def with_name(self, name):
        if name is None or not isinstance(name, str):
            raise TypeError("Incorrect name. Should be of type str")
        self.name = name
        return self

    def with_field(self, field):
        if field is None or not isinstance(field, str):
            raise TypeError("Incorrect field. Should be of type str")
        self.field = field
        return self

    def build(self):
        gdpr_field_request = GdprFieldRequestBuilder.GdprFieldRequest(self.list_id, self.field, self.name,
                                                                      self.description, self.etag)
        return gdpr_field_request

    class GdprFieldRequest(AbstractJsonSerializable, AbstractHttpEtag):

        def __init__(self, list_id, field, name, description, etag):
            self.list_id = list_id
            self.field = field
            self.name = name
            self.description = description
            self.etag = etag

        # override
        def get_etag(self):
            return self.etag

        # override
        def get_body(self):

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)
            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
