import json

from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable
from Apifon.sdk.Response.SubgroupResponse import SubgroupResponse


class SubgroupRequestBuilder:
    def __init__(self, list_id, group_field, name, option_field):
        self.list_id = list_id
        self.group_field = group_field
        self.name = name
        self.option_field = option_field

    @staticmethod
    def get_add_request_builder(name, list_id, group_field):
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if group_field is None or not isinstance(group_field, str):
            raise TypeError("Invalid group field. Should be of type str")

        return SubgroupRequestBuilder(list_id, group_field, name, None)

    @staticmethod
    def get_add_multiple_request_builder(names, list_id, group_field):
        if names is None or not isinstance(names, list):
            raise TypeError("Invalid name. Should be of type list")
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if group_field is None or not isinstance(group_field, str):
            raise TypeError("Invalid group field. Should be of type str")

        return SubgroupRequestBuilder(list_id, group_field, names, None)

    @staticmethod
    def get_update_request_builder(name, list_id, group_field, options_field):
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if group_field is None or not isinstance(group_field, str):
            raise TypeError("Invalid group field. Should be of type str")
        if options_field is None or not isinstance(options_field, str):
            raise TypeError("Invalid subgroup field. Should be of type str")

        return SubgroupRequestBuilder(list_id, group_field, name, options_field)

    def build(self):
        return self.SubgroupRequest(self.name,self.list_id,self.group_field,self.option_field)


    class SubgroupRequest(AbstractJsonSerializable):

        def __init__(self, name, list_id, group_field, options_field):
            self.list_id = list_id
            self.group_field = group_field
            self.name = name
            self.options_field = options_field

        # override
        def get_body(self):

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)
            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif var[i] == self.list_id:
                    var.pop(i)
                elif var[i] == self.group_field:
                    var.pop(i)
                elif var[i] == self.options_field:
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
