import json

from Apifon.sdk.Resource.Helpers.AbstractFieldSerializable import AbstractFieldSerializable
from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable
from Apifon.sdk.Resource.Helpers.Condition import Condition
from Apifon.sdk.Resource.Helpers.ConditionOperator import ConditionOperator
from Apifon.sdk.Resource.Helpers.SortDirection import SortDirection
from Apifon.sdk.Model.Sort import Sort


class SubscriberConditionQueryRequestBuilder(AbstractFieldSerializable):

    def __init__(self, list_id):
        self.list_id = list_id
        self.page = 1
        self.size = 20
        self.conditions = list()
        self.fields = set()
        self.sort = Sort.get_default_sort()

    @staticmethod
    def get_subscriber_conditions_builder(list_id):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        return SubscriberConditionQueryRequestBuilder(list_id)

    def clear_conditions(self):
        self.conditions.clear()
        return self

    def with_condition_parameters(self, name, operator, value):
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        if operator is None or not ConditionOperator.has_value(operator):
            raise TypeError("Invalid operator. Should be of type ConditionOperator.value")
        if value is None:
            raise ValueError("value is empty")
        self.conditions.append(Condition(name, operator, value))
        return self

    def with_condition(self, condition):
        if condition is None or not isinstance(condition, Condition):
            raise TypeError("Invalid condition. Should be of type Condition")
        if condition.name is None or not isinstance(condition.name, str):
            raise TypeError("Invalid condition.name. Should be of type str")
        if condition.op is None or not ConditionOperator.has_value(condition.op):
            raise TypeError("Invalid condition.operator. Should be of type ConditionOperator.value")
        if condition.vl is None:
            raise ValueError("condition.value is empty")
        self.conditions.append(condition)
        return self

    def with_conditions(self, conditions):
        if conditions is None or not isinstance(conditions, list):
            raise TypeError("Invalid conditions. Should be of type list")
        for condition in conditions:
            try:
                self.with_condition(condition)
            except Exception:
                raise
        return self

    def with_page(self, page):
        if page is None or not isinstance(page, int):
            raise TypeError("Invalid page. Should be of type int")
        self.page = page
        return self

    def with_size(self, size):
        if size is None or not isinstance(size, int):
            raise TypeError("Invalid size. Should be of type int")
        self.size = size
        return self

    def with_sort(self, sort):
        if sort is None or not isinstance(sort, Sort):
            raise TypeError("Invalid sort. Should be of type Sort")
        self.sort = sort
        return self

    def with_sort_parameters(self, sort_direction, sort_field):
        if sort_direction is None or not SortDirection.has_value(sort_direction):
            raise TypeError("Invalid sort direction. Should be of type SortDirection.value")
        if sort_field is None or not isinstance(sort_field, str):
            raise TypeError("Invalid sort field. Should be of type str")
        self.sort = Sort(sort_direction, sort_field)
        return self

    def add_visible_fields(self, fields):
        if fields is None or not isinstance(fields, set):
            raise TypeError("Invalid fields. Should be of type Set ")
        self.fields.update(fields)
        return self

    def add_visible_field(self, field):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        self.fields.add(field)
        return self

    def get_fields(self):
        return self.fields

    def get_path_id(self):
        return self.list_id

    def build(self):
        return SubscriberConditionQueryRequestBuilder.SubscriberConditionQueryRequest(self.list_id, self.page,
                                                                                      self.size,
                                                                                      self.conditions, self.fields,
                                                                                      self.sort)

    class SubscriberConditionQueryRequest(AbstractJsonSerializable):

        def __init__(self, list_id, page, size, conditions, fields, sort):
            self.list_id = list_id
            self.page = page
            self.size = size
            self.conditions = conditions
            self.fields = list(fields)
            self.sort = sort

        # override
        def get_body(self):

            if not bool(self.conditions):
                self.conditions = None
            if not bool(self.fields):
                self.fields = None

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)
            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
