import json

from Apifon.sdk.Resource.Helpers.AbstractHttpEtag import AbstractHttpEtag
from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable
from Apifon.sdk.Resource.Helpers.SubscriberFieldDateFormat import SubscriberFieldDateFormat
from Apifon.sdk.Resource.Helpers.SubscriberFieldType import SubscriberFieldType
from Apifon.sdk.Response.SubscriberFieldResponse import SubscriberFieldResponse


class SubscriberFieldRequestBuilder(AbstractHttpEtag):

    def __init__(self, id_, length, name, type_=None, field=None):
        self.id = id_
        self.field = field
        self.etag = None
        self.length = length
        self.name = name
        self.type = type_
        self.date_format = None
        self.internal = False

    @staticmethod
    def get_update_subscriber_field_request_builder(list_id, field, name, length):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list. Should be of type str")
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        if length is None or not isinstance(length, int):
            raise TypeError("Invalid length. Should be of type int")

        sub_filed_request_builder = SubscriberFieldRequestBuilder(list_id, length, name, None, field)
        return sub_filed_request_builder

    @staticmethod
    def get_update_subscriber_field_request_builder_from_response(list_id, subscriber_field_response):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if subscriber_field_response is None or not isinstance(subscriber_field_response, SubscriberFieldResponse):
            raise TypeError("Invalid subscriber_field_response. Should be of type SubscriberFieldResponse")

        sub_filed_request_builder = SubscriberFieldRequestBuilder(list_id, subscriber_field_response.length,
                                                                  subscriber_field_response.name,
                                                                  subscriber_field_response.type,
                                                                  subscriber_field_response.field)
        sub_filed_request_builder.date_format = subscriber_field_response.date_format
        sub_filed_request_builder.internal = sub_filed_request_builder.internal

        return sub_filed_request_builder

    #type should be of SubscriberFieldType.value
    @staticmethod
    def get_add_request_builder(list_id, type_, length, name):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if type_ is None or not SubscriberFieldType.has_value(type_):
            raise TypeError("Invalid type. Should be of type SubscriberFieldType.value")
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        if length is None or not isinstance(length, int):
            raise TypeError("Invalid length. Should be of type int")

        sub_filed_request_builder = SubscriberFieldRequestBuilder(list_id, length, name, type_)
        return sub_filed_request_builder

    def with_date_format(self, date_format):
        if date_format is None or not SubscriberFieldDateFormat.has_value(date_format):
            raise TypeError("Invalid date. Should be of type SubscriberFieldDateFormat.value")
        self.date_format = date_format
        return self

    def with_internal(self, internal):
        if internal is None or not isinstance(internal, bool):
            raise TypeError("Invalid internal, Should be of type bool")
        self.internal = internal
        return self

    def with_name(self, name):
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        self.name = name
        return self

    def with_length(self, length):
        if length is None or not isinstance(length, int):
            raise TypeError("Invalid length. Should be of type int")
        self.length = length
        return self

    def with_list_etag(self, list_etag):
        if list_etag is None or not isinstance(list_etag, str):
            raise TypeError("Invalid list etag. Should be of type str")
        self.etag = list_etag
        return self

    def build(self):
        sub_field = SubscriberFieldRequestBuilder.SubscriberFieldRequest(self.id, self.field, self.etag, self.length,
                                                                         self.name, self.type, self.date_format,
                                                                         self.internal)
        return sub_field

    def get_etag(self):
        return self.etag

    class SubscriberFieldRequest(AbstractJsonSerializable, AbstractHttpEtag):

        def __init__(self, id_, field, etag, length, name, type_, date_format, internal):
            self.list_id = id_
            self.field = field
            self.etag = etag
            self.length = length
            self.name = name
            self.type = type_
            self.date_format = date_format
            self.internal = internal

        # override
        def get_etag(self):
            return self.etag

        # override
        def get_body(self):

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)
            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
