from urllib.parse import urlencode

from Apifon.sdk.Resource.Helpers.AbstractFieldSerializable import AbstractFieldSerializable
from Apifon.sdk.Resource.Helpers.PaginatedRequest import PaginatedRequest
from Apifon.sdk.Resource.Helpers.PaginatedRequestBuilder import PaginatedRequestBuilder
from Apifon.sdk.Resource.Helpers.SubscriberStatus import SubscriberStatus


class SubscriberQueryBuilder(PaginatedRequestBuilder, AbstractFieldSerializable):

    def __init__(self, list_id):
        super().__init__()
        self.list_id = list_id
        self.status = None
        self.email_status = None
        self.fields = set()

    @staticmethod
    def get_query_builder(list_id):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        return SubscriberQueryBuilder(list_id)

    #status should be of type SubscriberStatus.value
    def with_status(self, status):
        if status is None or not SubscriberStatus.has_value(status):
            raise TypeError("Invalid status. Should be of type SubscriberStatus.value")
        self.status = status
        return self

    # email status should be of type SubscriberStatus.value
    def with_email_status(self, email_status):
        if email_status is None or not SubscriberStatus.has_value(email_status):
            raise TypeError("Invalid email status. Should be of type SubscriberStatus.value")
        self.email_status = email_status
        return self

    def add_visible_fields(self, fields):
        if fields is None or not isinstance(fields, set):
            raise TypeError("Invalid fields. Should be of type Set ")
        self.fields.update(fields)
        return self

    def add_visible_field(self, field):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        self.fields.add(field)
        return self

    def get_fields(self):
        return self.fields

    def get_path_id(self):
        return self.list_id

    def to_query_param_request(self):
        return self.build().to_query_param_request()

    def build(self):
        return SubscriberQueryBuilder.SubscriberQueryRequest(self.list_id, self.status, self.email_status, self.fields,
                                                             self.size, self.page, self.sort, self.sort_dir)

    class SubscriberQueryRequest(PaginatedRequest):

        def __init__(self, list_id, status, email_status, fields, size, page, sort, sort_dir):
            super().__init__(page, size, sort, sort_dir)
            self.list_id = list_id
            self.status = status
            self.email_status = email_status
            self.fields = fields

        def to_query_param_request(self):
            queryParams = {}
            self.add_query_params(queryParams)
            if self.status is not None and SubscriberStatus.has_value(self.status):
                queryParams["status"] = self.status
            if self.email_status is not None and SubscriberStatus.has_value(self.email_status):
                queryParams["email_status"] = self.email_status
            if self.fields is not None and bool(self.fields):
                queryParams['fields'] = ','.join(self.fields)
            return '?' + urlencode(queryParams)

        def get_path_id(self):
            return self.list_id
