import json
import os
from urllib.parse import urlsplit, SplitResult

from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Model.CollectionEntityResponse import CollectionEntityResponse
from Apifon.sdk.Model.Request.BounceQueryBuilder import BounceQueryBuilder
from Apifon.sdk.Model.Request.BounceRequestBuilder import BounceRequestBuilder
from Apifon.sdk.Model.Request.BounceSingleQuery import BounceSingleQuery
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Resource.UriLocator import UriLocator
from Apifon.sdk.Response.BounceResponse import BounceResponse


class BounceListResource(AbstractResource, UriLocator):
    def __init__(self, endpoint="/services/api/v1/bounce/subscriber"):
        super().__init__(endpoint)

    def get_bounce_subscribers(self, paginated_request):
        if paginated_request is None \
                or not isinstance(paginated_request,
                                  BounceQueryBuilder.QueryBounceSubscribersRequest) \
                and not isinstance(paginated_request, SplitResult):
            raise TypeError("Invalid request. Must be of type QueryBounceSubscribersRequest or SplitResult")
        if isinstance(paginated_request, BounceQueryBuilder.QueryBounceSubscribersRequest):

            self.cur_method = "GET"
            self.cur_endpoint = self.endpoint + "/" + paginated_request.to_query_param_request()
            self.cur_body = ""
            response = self.dispatch_to_response()
            bounce_list = json.loads(response.read().decode('utf8'))
            return CollectionEntityResponse(bounce_list, 'bounce')

        if isinstance(paginated_request, SplitResult):
            return self.get_list_by_resource_uri(paginated_request)


    def get_bounce_subscriber(self, bounce_single_query):
        if bounce_single_query is None or not isinstance(bounce_single_query, BounceSingleQuery):
            raise TypeError("Invalid request. Must be of type BounceSingleQuery")

        self.cur_method = "GET"
        self.cur_endpoint = self.endpoint + "/" + bounce_single_query.get_path_id() + bounce_single_query.get_query_params()
        self.cur_body = ""
        response = self.dispatch_to_response()
        bounce_sub = json.loads(response.read().decode('utf8'))
        return BounceResponse(bounce_sub)


    def add_bounce_subscriber(self, bounce_request):
        if bounce_request is None or not isinstance(bounce_request, BounceRequestBuilder.BounceRequest):
            raise TypeError("Invalid bounce request. Should be of type BounceRequest")

        self.cur_method = "POST"
        self.cur_endpoint = self.endpoint + "/block"
        self.cur_body = bounce_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url


    def remove_bounce_subscriber(self, destination):
        if destination is None or not isinstance(destination, str):
            raise TypeError("Invalid destination. Should be of type str")

        self.cur_method = "DELETE"
        self.cur_endpoint = self.endpoint + "/unblock/" + destination
        self.cur_body = ""
        response = self.dispatch_to_response()
        return response.status == 204


    def get_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
            raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")

        destination = os.path.basename(os.path.normpath(resource_uri.path))
        return self.get_bounce_subscriber(BounceSingleQuery.get_bounce_lookup(destination))


    def get_list_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
            raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")

        self.cur_method = "GET"
        query_params = ""
        if resource_uri.query is not None and bool(resource_uri.query):
            query_params = "?" + resource_uri.query
        self.cur_endpoint = resource_uri.path + query_params
        self.cur_body = ""
        response = self.dispatch_to_response()
        bounce_list = json.loads(response.read().decode('utf8'))
        return CollectionEntityResponse(bounce_list, 'bounce')

