from urllib.parse import urlsplit

from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Model.Request.GdprFieldRequestBuilder import GdprFieldRequestBuilder
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Resource.UriLocator import UriLocator


class GdprFieldResource(AbstractResource, UriLocator):
    def __init__(self, endpoint="/services/api/v1/list/{list_id}/gdpr"):
        super().__init__(endpoint)

    def replace_placeholder_and_set_url(self, endpoint, list_id):
        self.cur_endpoint = endpoint.replace("{list_id}", list_id)

    def add_gdpr_field(self, gdpr_field_request):
        if not isinstance(gdpr_field_request, GdprFieldRequestBuilder.GdprFieldRequest):
            raise TypeError("Invalid gdpr_field_request. Should be of type "
                            "GdprFieldRequestBuilder.GdprFieldRequest")
        if gdpr_field_request.list_id is None or not bool(gdpr_field_request.list_id):
            raise ValueError("list id is null")

        self.cur_method = "POST"
        self.replace_placeholder_and_set_url(self.endpoint + "/", gdpr_field_request.list_id)
        # self.add_etag_header(gdpr_field_request.get_etag())
        self.cur_body = gdpr_field_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url


    def update_gdpr_field(self, gdpr_field_request):
        if not isinstance(gdpr_field_request, GdprFieldRequestBuilder.GdprFieldRequest):
            raise TypeError("Invalid gdpr_field_request. Should be of type "
                            "GdprFieldRequestBuilder.GdprFieldRequest")
        if gdpr_field_request.list_id is None or not bool(gdpr_field_request.list_id):
            raise ValueError("list id is null")

        self.cur_method = "PUT"
        self.replace_placeholder_and_set_url(self.endpoint + "/" + gdpr_field_request.field,
                                             gdpr_field_request.list_id)
        # self.add_etag_header(gdpr_field_request.get_etag())
        self.cur_body = gdpr_field_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url


    def get_by_resource_uri(self, resource_uri):
        raise NotImplementedError

    def get_list_by_resource_uri(self, resource_uri):
        raise NotImplementedError
