from datetime import datetime, timedelta


class OAuthCredentials:

    def __init__(self, array_values = None):
        self.access_token = None
        self.refresh_token = None
        self.access_code = None
        self.expires_in = None
        self.expiration_date = None
        self.assign_response(array_values)

    def assign_response(self, array_values):
        if array_values is not None:
            if 'access_token' in array_values:
                self.access_token = array_values['access_token']
            if 'expires_in' in array_values:
                self.expires_in = array_values['expires_in']

    @staticmethod
    def get_OAuth_Credentials_from_access_code(access_code):
        if access_code is None or not isinstance(access_code, str):
            raise TypeError("Invalid access_code. Should be of type str")
        oauth_credentials = OAuthCredentials()
        oauth_credentials.access_code = access_code
        return oauth_credentials

    def set_expires_in(self, expires_in):
        if expires_in is None or not isinstance(expires_in, int):
            raise TypeError("Invalid expires_in. Should be of type int")
        self.expires_in = expires_in
        timeNow = datetime.now()
        self.expiration_date = timeNow + timedelta(seconds=expires_in)

    def get_expiration_date(self):
        return self.expiration_date.time()

    def has_expired(self):
        if self.expiration_date is not None:
            return self.expiration_date.date() < datetime.today().date()
        return True

    def __str__(self):
        return "OAuthCredentials{" + "accessToken=" + str(self.access_token) + ", refreshToken=" + str(self.refresh_token) + \
               ", accessCode=" + str(self.access_code) + ", expiresIn=" + str(self.expires_in) + ", expirationDate=" + \
               str(self.expiration_date) + '}'
