import json
from urllib.parse import urlsplit

from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Model.Request.SubscriberFieldRequestBuilder import SubscriberFieldRequestBuilder
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Resource.UriLocator import UriLocator
from Apifon.sdk.Response.SubscriberFieldResponse import SubscriberFieldResponse


class SubscriberFieldResource(AbstractResource, UriLocator):

    def __init__(self, endpoint="/services/api/v1/list/{list_id}/field"):
        super().__init__(endpoint)

    def replace_placeholder_and_set_url(self, endpoint, list_id):
        self.cur_endpoint = endpoint.replace("{list_id}", list_id)

    def add_subscriber_field(self, subscriber_field_request):
        if not isinstance(subscriber_field_request, SubscriberFieldRequestBuilder.SubscriberFieldRequest):
            raise TypeError("Invalid subscriber_field_request. Should be of type "
                            "SubscriberFieldRequestBuilder.SubscriberFieldRequest")
        if subscriber_field_request.list_id is None or not bool(subscriber_field_request.list_id):
            raise ValueError("list id is null")

        self.cur_method = "POST"
        self.replace_placeholder_and_set_url(self.endpoint + "/", subscriber_field_request.list_id)
        # self.add_etag_header(subscriber_field_request.get_etag())
        self.cur_body = subscriber_field_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url


    def update_subscriber_field(self, subscriber_field_request):
        if not isinstance(subscriber_field_request, SubscriberFieldRequestBuilder.SubscriberFieldRequest):
            raise TypeError("Invalid subscriber_field_request. Should be of type "
                            "SubscriberFieldRequestBuilder.SubscriberFieldRequest")
        if subscriber_field_request.list_id is None or not bool(subscriber_field_request.list_id):
            raise ValueError("list id is null")

        self.cur_method = "PUT"
        newEndpoint = self.endpoint + "/" + subscriber_field_request.field
        self.replace_placeholder_and_set_url(newEndpoint, subscriber_field_request.list_id)
        # self.add_etag_header(subscriber_field_request.get_etag())
        self.cur_body = subscriber_field_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url


    def remove_subscriber_field(self, list_id, field):
        if list_id is None or not bool(list_id):
            raise ValueError("list id is null")
        if field is None or not bool(field):
            raise ValueError("field is null")

        self.cur_method = "DELETE"
        self.replace_placeholder_and_set_url(self.endpoint + "/" + field, list_id)
        self.cur_body = ""
        response = self.dispatch_to_response()
        return response.status == 204


    def get_by_resource_uri(self, resource_uri):
        raise NotImplementedError

    def get_list_by_resource_uri(self, resource_uri):
        raise NotImplementedError
