from datetime import datetime

from Apifon.sdk.Model.SingleResponse import SingleResponse
from Apifon.sdk.Model.Request.CampaignDefaultsBuilder import CampaignDefaultsBuilder
from Apifon.sdk.Model.Request.GdprPropertiesBuilder import GdprPropertiesBuilder
from Apifon.sdk.Response.GdprFieldResponse import GdprFieldResponse
from Apifon.sdk.Response.GroupResponse import GroupResponse
from Apifon.sdk.Response.SubscriberFieldResponse import SubscriberFieldResponse


class ListResponse(SingleResponse):
    def __init__(self, array_values):
        super().__init__(array_values)
        self.id = None
        self.account_id = None
        self.type = None
        self.title = None
        self.status = None
        self.description = None
        self.date_created = None
        self.date_updated = None
        self.campaign_defaults = None
        self.total_subscribers = None
        self.active_subscribers = None
        self.active_email_subscribers = None
        self.form_id = None
        self.subscriber_fields = []
        self.gdpr_fields = []
        self.brand_name = None
        self.company_name = None
        self.gdpr_properties = None
        self.form_urls = {}
        self.prefix = None
        self.groups = []
        self.assign_values(array_values)

    def assign_values(self, array_values):
        if 'id' in array_values:
            self.id = array_values['id']
        if 'account_id' in array_values:
            self.account_id = array_values['account_id']
        if 'type' in array_values:
            self.type = array_values['type']
        if 'title' in array_values:
            self.title = array_values['title']
        if 'status' in array_values:
            self.status = array_values['status']
        if 'description' in array_values:
            self.description = array_values['description']
        if 'date_created' in array_values:
            self.date_created = datetime.fromtimestamp(int(array_values['date_created'])/1000).strftime('%c')
        if 'date_updated' in array_values:
            self.date_updated = datetime.fromtimestamp(int(array_values['date_updated'])/1000).strftime('%c')
        if 'campaign_defaults' in array_values:
            self.campaign_defaults = self.__assign_campaign_defaults(array_values['campaign_defaults'])
        if 'total_subscribers' in array_values:
            self.total_subscribers = array_values['total_subscribers']
        if 'active_subscribers' in array_values:
            self.active_subscribers = array_values['active_subscribers']
        if 'active_email_subscribers' in array_values:
            self.active_email_subscribers = array_values['active_email_subscribers']
        if 'form_id' in array_values:
            self.form_id = array_values['form_id']
        if 'subscriber_fields' in array_values and bool(array_values['subscriber_fields']):
            for result in array_values['subscriber_fields']:
                self.subscriber_fields.append(SubscriberFieldResponse(result))
        if 'gdpr_fields' in array_values and bool(array_values['gdpr_fields']):
            for result in array_values['gdpr_fields']:
                self.gdpr_fields.append(GdprFieldResponse(result))
        if 'brand_name' in array_values:
            self.brand_name = array_values['brand_name']
        if 'company_name' in array_values:
            self.company_name = array_values['company_name']
        if 'gdpr_properties' in array_values:
            self.gdpr_properties = self.__assign_gdpr_properties(array_values['gdpr_properties'])
        if 'form_urls' in array_values and bool(array_values['form_urls']):
            for k, v in array_values['form_urls'].items():
                self.form_urls[k] = v
        if 'prefix' in array_values:
            self.prefix = array_values['prefix']
        if 'groups' in array_values:
            for result in array_values['groups']:
                self.groups.append(GroupResponse(result))

    @staticmethod
    def __assign_campaign_defaults(array_values):
        sender_id = None
        viber_sender_id = None
        sender_name = None
        from_ = None
        reply_to = None

        if 'sender_id' in array_values:
            sender_id = array_values['sender_id']
        if 'viber_sender_id' in array_values:
            viber_sender_id = array_values['viber_sender_id']
        if 'sender_name' in array_values:
            sender_name = array_values['sender_name']
        if 'from' in array_values:
            from_ = array_values['from']
        if 'reply_to' in array_values:
            reply_to = array_values['reply_to']

        campaign_defaults = CampaignDefaultsBuilder().CampaignDefaults(sender_id, viber_sender_id, sender_name, from_,
                                                                       reply_to)
        return campaign_defaults

    @staticmethod
    def __assign_gdpr_properties(array_values):
        gdpr_compliant = None
        title = None
        description = None
        legal_text = None

        if 'gdpr_compliant' in array_values:
            gdpr_compliant = array_values['gdpr_compliant']
        if 'title' in array_values:
            title = array_values['title']
        if 'description' in array_values:
            description = array_values['description']
        if 'legal_text' in array_values:
            legal_text = array_values['legal_text']

        gdpr_properties = GdprPropertiesBuilder().GdprProperties(gdpr_compliant, title, description, legal_text)
        return gdpr_properties
